import numpy as np


class GenPopVanilla:
    def gen_pop(self, sol_per_pop : int, n_features : int, select_n_features : int) -> list:
        pop = []
        for _ in range(sol_per_pop):
            pop.append(np.random.choice([i for i in range(n_features)], size=select_n_features, replace=False).tolist())
        return pop


if __name__ == "__main__":
    vanilla_pop = GenPopVanilla()
    pop = vanilla_pop.gen_pop(5, 50, 15)
    
    for p in pop:
        print(sorted(p))



from genetic_feature_selection.genetic_funcs import weighted_selection
import pytest


@pytest.fixture
def evaluted_feature_vectors():
    efvs = [
        (0.9, [1,2,3,4]),
        (0.88, [5,6,7,8]),
    ]
    return efvs


@pytest.fixture
def evaluted_feature_vectors_0_score():
    efvs = [
        (0.9, [1,2,3,4]),
        (0.0, [5,6,7,8]),
    ]
    return efvs


def test_weighted_selection(evaluted_feature_vectors):
    child = weighted_selection(
        evaluted_feature_vectors[0], evaluted_feature_vectors[1]
    )

    assert len(child) == len(evaluted_feature_vectors[0][1])


def test_weighted_selection_1_0(evaluted_feature_vectors_0_score):
    """When one of the parents have score equal to zero"""
    p1_genes = evaluted_feature_vectors_0_score[0][1] 

    child = weighted_selection(
        evaluted_feature_vectors_0_score[0], 
        evaluted_feature_vectors_0_score[1]
    )

    assert len(child) == len(evaluted_feature_vectors_0_score[0][1])




from dataclasses import dataclass
from typing import List
import datetime


@dataclass
class GeneticIter:
    iteration : int
    score : float
    cols : List[str]
    iter_scores : List[float]

    def __str__(self):
        return f"{datetime.datetime.now()} : | Iteration {self.iteration} | score {self.score} | cols {self.cols}"
